'use strict';

const fs = require('node:fs');
const path = require('node:path');
const parseArgs = require('minimist');
const index = require('./index.cjs');
require('@vue/compiler-sfc');
require('@babel/types');
require('ast-kit');
require('magic-string-ast');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  const n = Object.create(null);
  if (e) {
    for (const k in e) {
      n[k] = e[k];
    }
  }
  n.default = e;
  return n;
}

const fs__namespace = /*#__PURE__*/_interopNamespaceCompat(fs);
const path__namespace = /*#__PURE__*/_interopNamespaceCompat(path);
const parseArgs__default = /*#__PURE__*/_interopDefaultCompat(parseArgs);

async function read(stream) {
  const chunks = [];
  for await (const chunk of stream) {
    chunks.push(chunk);
  }
  return Buffer.concat(chunks).toString("utf8");
}
async function main() {
  const args = parseArgs__default(process.argv.slice(2));
  const inputFile = args._[0];
  const inputPromise = inputFile ? fs__namespace.promises.readFile(inputFile, "utf8") : read(process.stdin);
  let config;
  if (args.config) {
    const { default: importedConfig } = await import(path__namespace.resolve(args.config));
    config = importedConfig;
  }
  const input = await inputPromise;
  const output = index.transformSFC(input, config);
  const outFile = args.print ? void 0 : args.out ?? inputFile;
  if (outFile) {
    await fs__namespace.promises.writeFile(outFile, output);
  } else {
    process.stdout.write(output + "\n");
  }
}
main();
